@@ -9,7 +9,6 @@ from dagster import (
     AssetIn,
     AssetSelection,
     Field,
-    FileHandle,
     In,
     Int,
     List,
@@ -19,14 +18,15 @@ from dagster import (
     asset,
     define_asset_job,
     fs_io_manager,
+    graph,
     job,
+    mem_io_manager,
+    op,
     repository,
     resource,
     with_resources,
 )
 from dagster._core.definitions.utils import DEFAULT_OUTPUT
-from dagster._core.storage.file_manager import local_file_manager
-from dagster._legacy import InputDefinition, ModeDefinition, composite_solid, pipeline, solid
 from dagster._utils import PICKLE_PROTOCOL, file_relative_path
 
 try:
@@ -66,41 +66,30 @@ def nb_test_path(name):
 def test_nb_op(name, **kwargs):
     outs = kwargs.pop("outs", {DEFAULT_OUTPUT: Out(is_required=False)})
     path = kwargs.pop("path", nb_test_path(name))
+    output_nb_name = kwargs.pop("output_notebook_name", "notebook")
 
     return dagstermill.define_dagstermill_op(
         name=name,
         notebook_path=path,
-        output_notebook_name="notebook",
+        output_notebook_name=output_nb_name,
         outs=outs,
         **kwargs,
     )
 
 
-default_mode_defs = [
-    ModeDefinition(
-        resource_defs={
-            "output_notebook_io_manager": local_output_notebook_io_manager,
-            "io_manager": fs_io_manager,
-        }
-    )
-]
-
+common_resource_defs = {
+    "output_notebook_io_manager": local_output_notebook_io_manager,
+    "io_manager": fs_io_manager,  # TODO check if necessary
+}
 
 hello_world = test_nb_op("hello_world", outs={})
 
 
-@pipeline(mode_defs=default_mode_defs)
-def hello_world_pipeline():
+@job(resource_defs=common_resource_defs)
+def hello_world_job():
     hello_world()
 
 
-hello_world_op = test_nb_op(
-    "hello_world_op",
-    path=nb_test_path("hello_world"),
-    outs={},
-)
-
-
 def build_hello_world_job():
     @job(
         resource_defs={
@@ -108,22 +97,22 @@ def build_hello_world_job():
         }
     )
     def hello_world_job():
-        hello_world_op()
+        hello_world()
 
     return hello_world_job
 
 
-hello_world_with_custom_tags_and_description = dagstermill.factory.define_dagstermill_solid(
+hello_world_with_custom_tags_and_description = test_nb_op(
     name="hello_world_custom",
-    notebook_path=nb_test_path("hello_world"),
+    path=nb_test_path("hello_world"),
     output_notebook_name="notebook",
     tags={"foo": "bar"},
     description="custom description",
 )
 
 
-@pipeline(mode_defs=default_mode_defs)
-def hello_world_with_custom_tags_and_description_pipeline():
+@job(resource_defs=common_resource_defs)
+def hello_world_with_custom_tags_and_description_job():
     hello_world_with_custom_tags_and_description()
 
 
@@ -133,64 +122,59 @@ hello_world_config = test_nb_op(
 )
 
 
-goodbye_config = dagstermill.factory.define_dagstermill_solid(
+goodbye_config = test_nb_op(
     name="goodbye_config",
-    notebook_path=nb_test_path("print_dagstermill_context_solid_config"),
+    path=nb_test_path("print_dagstermill_context_op_config"),
     output_notebook_name="notebook",
     config_schema={"farewell": Field(String, is_required=False, default_value="goodbye")},
 )
 
 
-@pipeline(mode_defs=default_mode_defs)
-def hello_world_config_pipeline():
+@job(resource_defs=common_resource_defs)
+def hello_world_config_job():
     hello_world_config()
     goodbye_config()
 
 
-@pipeline(mode_defs=default_mode_defs)
-def alias_config_pipeline():
+@job(resource_defs=common_resource_defs)
+def alias_config_job():
     hello_world_config.alias("aliased_greeting")()
     goodbye_config.alias("aliased_goodbye")()
 
 
-@solid(input_defs=[InputDefinition("notebook")])
+@op(ins={"notebook": In()})
 def load_notebook(notebook):
     return notebook
 
 
-@pipeline(mode_defs=default_mode_defs)
-def hello_world_with_output_notebook_pipeline():
+@job(resource_defs=common_resource_defs)
+def hello_world_with_output_notebook_job():
     notebook = hello_world()
     load_notebook(notebook)
 
 
-hello_world_no_output_notebook_no_file_manager = dagstermill.factory.define_dagstermill_solid(
-    name="hello_world_no_output_notebook_no_file_manager",
-    notebook_path=nb_test_path("hello_world"),
-)
-
-
-@pipeline
-def hello_world_no_output_notebook_no_file_manager_pipeline():
-    hello_world_no_output_notebook_no_file_manager()
-
-
-hello_world_no_output_notebook = dagstermill.factory.define_dagstermill_solid(
+hello_world_no_output_notebook = test_nb_op(
     name="hello_world_no_output_notebook",
-    notebook_path=nb_test_path("hello_world"),
+    path=nb_test_path("hello_world"),
+    output_notebook_name=None,
 )
 
 
-@pipeline(mode_defs=default_mode_defs)
-def hello_world_no_output_notebook_pipeline():
+@job
+def hello_world_no_output_notebook_no_file_manager_job():
+    hello_world_no_output_notebook()
+
+
+@job(resource_defs=common_resource_defs)
+def hello_world_no_output_notebook_job():
     hello_world_no_output_notebook()
 
 
 hello_world_output = test_nb_op("hello_world_output", outs={DEFAULT_OUTPUT: Out(str)})
 
 
-@pipeline(mode_defs=default_mode_defs)
-def hello_world_output_pipeline():
+@job(resource_defs=common_resource_defs)
+def hello_world_output_job():
     hello_world_output()
 
 
@@ -199,16 +183,16 @@ hello_world_explicit_yield = test_nb_op(
 )
 
 
-@pipeline(mode_defs=default_mode_defs)
-def hello_world_explicit_yield_pipeline():
+@job(resource_defs=common_resource_defs)
+def hello_world_explicit_yield_job():
     hello_world_explicit_yield()
 
 
 hello_logging = test_nb_op("hello_logging")
 
 
-@pipeline(mode_defs=default_mode_defs)
-def hello_logging_pipeline():
+@job(resource_defs=common_resource_defs)
+def hello_logging_job():
     hello_logging()
 
 
@@ -224,55 +208,55 @@ mult_two_numbers = test_nb_op(
 )
 
 
-@solid
+@op
 def return_one():
     return 1
 
 
-@solid
+@op
 def return_two():
     return 2
 
 
-@solid
+@op
 def return_three():
     return 3
 
 
-@solid
+@op
 def return_four():
     return 4
 
 
-@pipeline(mode_defs=default_mode_defs)
-def add_pipeline():
+@job(resource_defs=common_resource_defs)
+def add_job():
     add_two_numbers(return_one(), return_two())
 
 
-@pipeline(mode_defs=default_mode_defs)
-def double_add_pipeline():
+@job(resource_defs=common_resource_defs)
+def double_add_job():
     add_two_numbers.alias("add_two_numbers_1")(return_one(), return_two())
     add_two_numbers.alias("add_two_numbers_2")(return_three(), return_four())
 
 
-@solid(input_defs=[], config_schema=Int)
+@op(config_schema=Int)
 def load_constant(context):
-    return context.solid_config
+    return context.op_config
 
 
-@pipeline(mode_defs=default_mode_defs)
-def notebook_dag_pipeline():
+@job(resource_defs=common_resource_defs)
+def notebook_dag_job():
     a = load_constant.alias("load_a")()
     b = load_constant.alias("load_b")()
     num, _ = add_two_numbers(a, b)
     mult_two_numbers(num, b)
 
 
-error_notebook = test_nb_op("error_notebook")
+error_notebook = test_nb_op("error_notebook", save_notebook_on_failure=True)
 
 
-@pipeline(mode_defs=default_mode_defs)
-def error_pipeline():
+@job(resource_defs=common_resource_defs)
+def error_job():
     error_notebook()
 
 
@@ -285,17 +269,17 @@ if DAGSTER_PANDAS_PRESENT and SKLEARN_PRESENT and MATPLOTLIB_PRESENT:
 
     tutorial_RF = test_nb_op("tutorial_RF", ins={"df": In(DataFrame)})
 
-    @pipeline(mode_defs=default_mode_defs)
-    def tutorial_pipeline():
+    @job(resource_defs=common_resource_defs)
+    def tutorial_job():
         dfr, _ = clean_data()
         # FIXME get better names for these
         tutorial_LR(dfr)
         tutorial_RF(dfr)
 
 
-@solid("resource_solid", required_resource_keys={"list"})
-def resource_solid(context):
-    context.resources.list.append("Hello, solid!")
+@op(required_resource_keys={"list"})
+def resource_op(context):
+    context.resources.list.append("Hello, op!")
     return True
 
 
@@ -356,90 +340,73 @@ def filepicklelist_resource(init_context):
         filepicklelist.close()
 
 
-@pipeline(
-    mode_defs=[
-        ModeDefinition(
-            name="test",
-            resource_defs={
-                "list": ResourceDefinition(lambda _: []),
-                "io_manager": fs_io_manager,
-                "output_notebook_io_manager": local_output_notebook_io_manager,
-            },
-        ),
-        ModeDefinition(
-            name="prod",
-            resource_defs={
-                "list": filepicklelist_resource,
-                "output_notebook_io_manager": local_output_notebook_io_manager,
-                "io_manager": fs_io_manager,
-            },
-        ),
-    ]
+@job(
+    resource_defs={
+        "list": filepicklelist_resource,
+        "output_notebook_io_manager": local_output_notebook_io_manager,
+        "io_manager": fs_io_manager,
+    }
 )
-def resource_pipeline():
-    hello_world_resource(resource_solid())
+def resource_job():
+    hello_world_resource(resource_op())
 
 
-@pipeline(
-    mode_defs=[
-        ModeDefinition(
-            resource_defs={
-                "list": filepicklelist_resource,
-                "output_notebook_io_manager": local_output_notebook_io_manager,
-                "io_manager": fs_io_manager,
-            }
-        )
-    ]
+@job(
+    resource_defs={
+        "list": filepicklelist_resource,
+        "output_notebook_io_manager": local_output_notebook_io_manager,
+        "io_manager": fs_io_manager,
+    }
 )
-def resource_with_exception_pipeline():
-    hello_world_resource_with_exception(resource_solid())
+def resource_with_exception_job():
+    hello_world_resource_with_exception(resource_op())
 
 
 bad_kernel = test_nb_op("bad_kernel")
 
 
-@pipeline(mode_defs=default_mode_defs)
-def bad_kernel_pipeline():
+@job(resource_defs=common_resource_defs)
+def bad_kernel_job():
     bad_kernel()
 
 
 reimport = test_nb_op("reimport", ins={"l": In(List[int])}, outs={DEFAULT_OUTPUT: Out(int)})
 
 
-@solid
+@op
 def lister():
     return [1, 2, 3]
 
 
-@pipeline(mode_defs=default_mode_defs)
-def reimport_pipeline():
+@job(resource_defs=common_resource_defs)
+def reimport_job():
     reimport(lister())
 
 
 yield_3 = test_nb_op("yield_3", outs={DEFAULT_OUTPUT: Out(int)})
 
 
-@pipeline(mode_defs=default_mode_defs)
-def yield_3_pipeline():
+@job(resource_defs=common_resource_defs)
+def yield_3_job():
     yield_3()
 
 
 yield_obj = test_nb_op("yield_obj")
 
 
-@pipeline(mode_defs=default_mode_defs)
-def yield_obj_pipeline():
+@job(resource_defs=common_resource_defs)
+def yield_obj_job():
     yield_obj()
 
 
-@pipeline(mode_defs=default_mode_defs)
-def retries_pipeline():
+@job(resource_defs=common_resource_defs)
+def retries_job():
     test_nb_op("raise_retry")()
     test_nb_op("yield_retry")()
 
 
-@pipeline(mode_defs=default_mode_defs)
-def failure_pipeline():
+@job(resource_defs=common_resource_defs)
+def failure_job():
     test_nb_op("raise_failure")()
     test_nb_op("yield_failure")()
 
@@ -449,90 +416,51 @@ yield_something = test_nb_op(
 )
 
 
-@solid
+@op
 def fan_in(a, b):
     return f"{a} {b}"
 
 
-@pipeline(
-    mode_defs=[
-        ModeDefinition(
-            resource_defs={
-                "io_manager": fs_io_manager,
-                "output_notebook_io_manager": local_output_notebook_io_manager,
-            }
-        )
-    ]
-)
-def fan_in_notebook_pipeline():
-    val_a, _ = yield_something.alias("solid_1")()
-    val_b, _ = yield_something.alias("solid_2")()
+@job(resource_defs=common_resource_defs)
+def fan_in_notebook_job():
+    val_a, _ = yield_something.alias("op_1")()
+    val_b, _ = yield_something.alias("op_2")()
     fan_in(val_a, val_b)
 
 
-@pipeline(
-    mode_defs=[
-        ModeDefinition(
-            resource_defs={
-                "output_notebook_io_manager": local_output_notebook_io_manager,
-            }
-        )
-    ]
+@job(
+    resource_defs={
+        "output_notebook_io_manager": local_output_notebook_io_manager,
+        "io_manager": mem_io_manager,
+    }
 )
-def fan_in_notebook_pipeline_in_mem():
-    val_a, _ = yield_something.alias("solid_1")()
-    val_b, _ = yield_something.alias("solid_2")()
+def fan_in_notebook_job_in_mem():
+    val_a, _ = yield_something.alias("op_1")()
+    val_b, _ = yield_something.alias("op_2")()
     fan_in(val_a, val_b)
 
 
-@composite_solid
+@graph
 def outer():
     yield_something()
 
 
-@pipeline(
-    mode_defs=[
-        ModeDefinition(
-            resource_defs={
-                "io_manager": fs_io_manager,
-                "output_notebook_io_manager": local_output_notebook_io_manager,
-            }
-        )
-    ]
-)
-def composite_pipeline():
+@job(resource_defs=common_resource_defs)
+def graph_job():
     outer()
 
 
-###################################################################################################
-# Back compat
-###################################################################################################
-
-hello_world_legacy = dagstermill.factory.define_dagstermill_solid(
-    name="hello_world_legacy",
+custom_io_mgr_key_op = dagstermill.factory.define_dagstermill_op(
+    name="custom_io_mgr_op",
     notebook_path=nb_test_path("hello_world"),
-    output_notebook="notebook",
+    io_manager_key="my_custom_io_manager",
+    output_notebook_name="my_notebook",
 )
 
 
-@solid(input_defs=[InputDefinition("notebook", dagster_type=FileHandle)])
-def load_notebook_legacy(notebook):
-    return os.path.exists(notebook.path_desc)
-
-
-@pipeline(
-    mode_defs=[
-        ModeDefinition(
-            resource_defs={
-                "io_manager": fs_io_manager,
-                "file_manager": local_file_manager,
-            }
-        )
-    ]
-)
-def hello_world_with_output_notebook_pipeline_legacy():
-    notebook = hello_world_legacy()
-    load_notebook_legacy(notebook)
+@job(resource_defs={"my_custom_io_manager": local_output_notebook_io_manager})
+def custom_io_mgr_key_job():
+    custom_io_mgr_key_op()
 
 
 ###################################################################################################
@@ -560,7 +488,7 @@ hello_world_config_asset = dagstermill.define_dagstermill_asset(
 
 goodbye_config_asset = dagstermill.define_dagstermill_asset(
     name="goodbye_config_asset",
-    notebook_path=nb_test_path("print_dagstermill_context_solid_config"),
+    notebook_path=nb_test_path("print_dagstermill_context_op_config"),
     config_schema={"farewell": Field(String, is_required=False, default_value="goodbye")},
 )
 
@@ -568,6 +496,12 @@ hello_logging_asset = dagstermill.define_dagstermill_asset(
     name="hello_logging_asset", notebook_path=nb_test_path("hello_logging")
 )
 
+error_notebook_asset = dagstermill.define_dagstermill_asset(
+    name="error_notebook_asset",
+    notebook_path=nb_test_path("error_notebook"),
+    save_notebook_on_failure=True,
+)
+
 
 @asset
 def a():
@@ -581,7 +515,7 @@ def b():
 
 add_two_number_asset = dagstermill.define_dagstermill_asset(
     name="add_two_numbers_asset",
-    notebook_path=nb_test_path("add_two_numbers"),
+    notebook_path=nb_test_path("add_two_numbers_no_yield"),
     ins={"a": AssetIn("a"), "b": AssetIn("b")},
 )
 
@@ -591,6 +525,16 @@ hello_world_resource_asset = dagstermill.define_dagstermill_asset(
     required_resource_keys={"list"},
 )
 
+custom_io_mgr_key_asset = dagstermill.define_dagstermill_asset(
+    name="custom_io_mgr_key",
+    notebook_path=nb_test_path("hello_world"),
+    io_manager_key="my_custom_io_manager",
+)
+
+yield_event_asset = dagstermill.define_dagstermill_asset(
+    name="yield_event_asset", notebook_path=nb_test_path("yield_event")
+)
+
 
 # this is hacky. We need a ReconstructablePipeline to run dagstermill, and
 # ReconstructablePipeline.for_module() find the jobs defined in this file. So we need to resolve all
@@ -606,6 +550,8 @@ assets = with_resources(
         b,
         add_two_number_asset,
         hello_world_resource_asset,
+        error_notebook_asset,
+        yield_event_asset,
     ],
     resource_defs={
         "list": ResourceDefinition(lambda _: []),
@@ -629,118 +575,37 @@ goodbye_config_asset_job = make_resolved_job(goodbye_config_asset)
 hello_logging_asset_job = make_resolved_job(hello_logging_asset)
 add_two_number_asset_job = make_resolved_job(add_two_number_asset)
 hello_world_resource_asset_job = make_resolved_job(hello_world_resource_asset)
-
-
-# hello_world_asset_job = define_asset_job(name="hello_world_asset_job", selection=AssetSelection.keys("hello_world_asset")).resolve(assets, [])
+error_notebook_asset_job = make_resolved_job(error_notebook_asset)
+yield_event_asset_job = make_resolved_job(yield_event_asset)
 
 
 @repository
 def notebook_repo():
-    pipelines = [
-        bad_kernel_pipeline,
-        error_pipeline,
-        hello_world_pipeline,
-        hello_world_with_custom_tags_and_description_pipeline,
-        hello_world_config_pipeline,
-        hello_world_explicit_yield_pipeline,
-        hello_world_output_pipeline,
-        hello_world_with_output_notebook_pipeline,
-        hello_logging_pipeline,
-        resource_pipeline,
-        resource_with_exception_pipeline,
-        add_pipeline,
-        notebook_dag_pipeline,
-        reimport_pipeline,
-        yield_3_pipeline,
-        yield_obj_pipeline,
-        retries_pipeline,
-        failure_pipeline,
-        fan_in_notebook_pipeline_in_mem,
-        fan_in_notebook_pipeline,
-        hello_world_no_output_notebook_no_file_manager_pipeline,
-        hello_world_with_output_notebook_pipeline_legacy,
+    repo = [
+        bad_kernel_job,
+        error_job,
+        hello_world_job,
+        hello_world_with_custom_tags_and_description_job,
+        hello_world_config_job,
+        hello_world_explicit_yield_job,
+        hello_world_output_job,
+        hello_world_with_output_notebook_job,
+        hello_logging_job,
+        resource_job,
+        resource_with_exception_job,
+        add_job,
+        notebook_dag_job,
+        reimport_job,
+        yield_3_job,
+        yield_obj_job,
+        retries_job,
+        failure_job,
+        fan_in_notebook_job_in_mem,
+        fan_in_notebook_job,
+        hello_world_no_output_notebook_no_file_manager_job,
         *assets,
     ]
     if DAGSTER_PANDAS_PRESENT and SKLEARN_PRESENT and MATPLOTLIB_PRESENT:
-        pipelines += [tutorial_pipeline]
+        repo += [tutorial_job]
 
-    return pipelines
-
-
-###################################################################################################
-# Assets
-###################################################################################################
-
-# hello_world_asset = dagstermill.define_dagstermill_asset(
-#     name="hello_world_asset", notebook_path=nb_test_path("hello_world")
-# )
-
-# hello_world_with_custom_tags_and_description_asset = dagstermill.define_dagstermill_asset(
-#     name="hello_world_custom_asset",
-#     notebook_path=nb_test_path("hello_world"),
-#     op_tags={"foo": "bar"},
-#     description="custom description",
-# )
-
-# hello_world_config_asset = dagstermill.define_dagstermill_asset(
-#     "hello_world_config_asset",
-#     notebook_path=nb_test_path("hello_world_config"),
-#     config_schema={"greeting": Field(String, is_required=False, default_value="hello")},
-# )
-
-
-# goodbye_config_asset = dagstermill.define_dagstermill_asset(
-#     name="goodbye_config_asset",
-#     notebook_path=nb_test_path("print_dagstermill_context_solid_config"),
-#     config_schema={"farewell": Field(String, is_required=False, default_value="goodbye")},
-# )
-
-# hello_logging_asset = dagstermill.define_dagstermill_asset(
-#     name="hello_logging_asset", notebook_path=nb_test_path("hello_logging")
-# )
-
-
-# @asset
-# def a():
-#     return 1
-
-
-# @asset
-# def b():
-#     return 2
-
-
-# add_two_number_asset = dagstermill.define_dagstermill_asset(
-#     name="add_two_numbers_asset",
-#     notebook_path=nb_test_path("add_two_numbers"),
-#     ins={"a": AssetIn("a"), "b": AssetIn("b")},
-# )
-
-# hello_world_resource_asset = dagstermill.define_dagstermill_asset(
-#     "hello_world_resource_asset",
-#     notebook_path=nb_test_path("hello_world_resource"),
-#     required_resource_keys={"list"},
-# )
-
-
-# @repository
-# def notebook_assets_repo():
-#     return [*with_resources(
-#             [
-#                 hello_world_asset,
-#                 hello_world_with_custom_tags_and_description_asset,
-#                 hello_world_config_asset,
-#                 goodbye_config_asset,
-#                 hello_logging_asset,
-#                 a,
-#                 b,
-#                 add_two_number_asset,
-#                 hello_world_resource_asset,
-#             ],
-#             resource_defs={
-#                 "list": ResourceDefinition(lambda _: []),
-#                 "output_notebook_io_manager": local_output_notebook_io_manager,
-#             },
-#         ),
-#         define_asset_job(name="hello_world_asset_job", selection=AssetSelection.keys("hello_world_asset"))
-#     ]
+    return repo
