@@ -4,7 +4,7 @@ import pickle
 import sys
 import tempfile
 import uuid
-from typing import Any, Dict, List, Mapping, Optional, Sequence, Set, Union, cast
+from typing import Any, Mapping, Optional, Sequence, Set, Union, cast
 
 import nbformat
 import papermill
@@ -22,12 +22,9 @@ from dagster._core.execution.context.compute import SolidExecutionContext
 from dagster._core.execution.context.input import build_input_context
 from dagster._core.execution.context.system import StepExecutionContext
 from dagster._core.execution.plan.outputs import StepOutputHandle
-from dagster._core.storage.file_manager import FileHandle
-from dagster._legacy import InputDefinition, OutputDefinition, SolidDefinition
 from dagster._serdes import pack_value
 from dagster._seven import get_system_temp_directory
 from dagster._utils import mkdir_p, safe_tempfile_path
-from dagster._utils.backcompat import rename_warning
 from dagster._utils.error import serializable_error_info_from_exc_info
 
 from .compat import ExecutionError
@@ -113,12 +110,7 @@ def get_papermill_parameters(step_context, inputs, output_log_path, compute_desc
     mkdir_p(marshal_dir)
 
     if not isinstance(step_context.pipeline, ReconstructablePipeline):
-        if compute_descriptor == "solid":
-            raise DagstermillError(
-                "Can't execute a dagstermill solid from a pipeline that is not reconstructable. "
-                "Use the reconstructable() function if executing from python"
-            )
-        elif compute_descriptor == "asset":
+        if compute_descriptor == "asset":
             raise DagstermillError(
                 "Can't execute a dagstermill asset that is not reconstructable. "
                 "Use the reconstructable() function if executing from python"
@@ -160,6 +152,7 @@ def _dm_compute(
     output_notebook_name=None,
     asset_key_prefix=None,
     output_notebook=None,
+    save_notebook_on_failure=False,
 ):
     check.str_param(name, "name")
     check.str_param(notebook_path, "notebook_path")
@@ -189,9 +182,7 @@ def _dm_compute(
 
                 # Scaffold the registration here
                 nb = load_notebook_node(notebook_path)
-                compute_descriptor = (
-                    "solid" if dagster_factory_name == "define_dagstermill_solid" else "op"
-                )
+                compute_descriptor = "op"
                 nb_no_parameters = replace_parameters(
                     step_execution_context,
                     nb,
@@ -229,6 +220,17 @@ def _dm_compute(
                             "with RetryRequested or Failure to trigger their behavior."
                         )
 
+                    if save_notebook_on_failure:
+                        storage_dir = step_context.instance.storage_directory()
+                        storage_path = os.path.join(storage_dir, f"{prefix}-out.ipynb")
+                        with open(storage_path, "wb") as dest_file_obj:
+                            with open(executed_notebook_path, "rb") as obj:
+                                dest_file_obj.write(obj.read())
+
+                        step_execution_context.log.info(
+                            f"Failed notebook written to {storage_path}"
+                        )
+
                     raise
 
             step_execution_context.log.debug(
@@ -238,7 +240,7 @@ def _dm_compute(
                 )
             )
             if output_notebook_name is not None:
-                # yield output notebook binary stream as a solid output
+                # yield output notebook binary stream as an op output
                 with open(executed_notebook_path, "rb") as fd:
                     yield Output(fd.read(), output_notebook_name)
 
@@ -317,124 +319,19 @@ def _dm_compute(
     return _t_fn
 
 
-def define_dagstermill_solid(
-    name: str,
-    notebook_path: str,
-    input_defs: Optional[Sequence[InputDefinition]] = None,
-    output_defs: Optional[Sequence[OutputDefinition]] = None,
-    config_schema: Optional[Union[Any, Dict[str, Any]]] = None,
-    required_resource_keys: Optional[Set[str]] = None,
-    output_notebook: Optional[str] = None,
-    output_notebook_name: Optional[str] = None,
-    asset_key_prefix: Optional[Union[List[str], str]] = None,
-    description: Optional[str] = None,
-    tags: Optional[Dict[str, Any]] = None,
-):
-    """Wrap a Jupyter notebook in a solid.
-
-    Arguments:
-        name (str): The name of the solid.
-        notebook_path (str): Path to the backing notebook.
-        input_defs (Optional[List[InputDefinition]]): The solid's inputs.
-        output_defs (Optional[List[OutputDefinition]]): The solid's outputs. Your notebook should
-            call :py:func:`~dagstermill.yield_result` to yield each of these outputs.
-        required_resource_keys (Optional[Set[str]]): The string names of any required resources.
-        output_notebook (Optional[str]): If set, will be used as the name of an injected output of
-            type :py:class:`~dagster.FileHandle` that will point to the executed notebook (in
-            addition to the :py:class:`~dagster.AssetMaterialization` that is always created). This
-            respects the :py:class:`~dagster._core.storage.file_manager.FileManager` configured on
-            the pipeline resources via the "file_manager" resource key, so, e.g.,
-            if :py:class:`~dagster_aws.s3.s3_file_manager` is configured, the output will be a :
-            py:class:`~dagster_aws.s3.S3FileHandle`.
-        output_notebook_name: (Optional[str]): If set, will be used as the name of an injected output
-            of type of :py:class:`~dagster.BufferedIOBase` that is the file object of the executed
-            notebook (in addition to the :py:class:`~dagster.AssetMaterialization` that is always
-            created). It allows the downstream solids to access the executed notebook via a file
-            object.
-        asset_key_prefix (Optional[Union[List[str], str]]): If set, will be used to prefix the
-            asset keys for materialized notebooks.
-        description (Optional[str]): If set, description used for solid.
-        tags (Optional[Dict[str, str]]): If set, additional tags used to annotate solid.
-            Dagster uses the tag keys `notebook_path` and `kind`, which cannot be
-            overwritten by the user.
-
-    Returns:
-        :py:class:`~dagster.SolidDefinition`
-    """
-    check.str_param(name, "name")
-    check.str_param(notebook_path, "notebook_path")
-    input_defs = check.opt_list_param(input_defs, "input_defs", of_type=InputDefinition)
-    output_defs = check.opt_list_param(output_defs, "output_defs", of_type=OutputDefinition)
-    required_resource_keys = set(
-        check.opt_set_param(required_resource_keys, "required_resource_keys", of_type=str)
-    )
-
-    extra_output_defs = []
-    if output_notebook_name is not None:
-        required_resource_keys.add("output_notebook_io_manager")
-        extra_output_defs.append(
-            OutputDefinition(name=output_notebook_name, io_manager_key="output_notebook_io_manager")
-        )
-    # backcompact
-    if output_notebook is not None:
-        rename_warning(
-            new_name="output_notebook_name",
-            old_name="output_notebook",
-            breaking_version="0.14.0",
-        )
-        required_resource_keys.add("file_manager")
-        extra_output_defs.append(OutputDefinition(dagster_type=FileHandle, name=output_notebook))
-
-    if isinstance(asset_key_prefix, str):
-        asset_key_prefix = [asset_key_prefix]
-
-    asset_key_prefix = check.opt_list_param(asset_key_prefix, "asset_key_prefix", of_type=str)
-
-    default_description = f"This solid is backed by the notebook at {notebook_path}"
-    description = check.opt_str_param(description, "description", default=default_description)
-
-    user_tags = validate_tags(tags)
-    if tags is not None:
-        check.invariant(
-            "notebook_path" not in tags,
-            "user-defined solid tags contains the `notebook_path` key, but the `notebook_path` key is reserved for use by Dagster",
-        )
-        check.invariant(
-            "kind" not in tags,
-            "user-defined solid tags contains the `kind` key, but the `kind` key is reserved for use by Dagster",
-        )
-    default_tags = {"notebook_path": notebook_path, "kind": "ipynb"}
-
-    return SolidDefinition(
-        name=name,
-        input_defs=input_defs,
-        compute_fn=_dm_compute(
-            "define_dagstermill_solid",
-            name,
-            notebook_path,
-            output_notebook_name,
-            asset_key_prefix=asset_key_prefix,
-            output_notebook=output_notebook,  # backcompact
-        ),
-        output_defs=output_defs + extra_output_defs,
-        config_schema=config_schema,
-        required_resource_keys=required_resource_keys,
-        description=description,
-        tags={**user_tags, **default_tags},
-    )
-
-
 def define_dagstermill_op(
     name: str,
     notebook_path: str,
     ins: Optional[Mapping[str, In]] = None,
     outs: Optional[Mapping[str, Out]] = None,
-    config_schema: Optional[Union[Any, Dict[str, Any]]] = None,
+    config_schema: Optional[Union[Any, Mapping[str, Any]]] = None,
     required_resource_keys: Optional[Set[str]] = None,
     output_notebook_name: Optional[str] = None,
-    asset_key_prefix: Optional[Union[List[str], str]] = None,
+    asset_key_prefix: Optional[Union[Sequence[str], str]] = None,
     description: Optional[str] = None,
-    tags: Optional[Dict[str, Any]] = None,
+    tags: Optional[Mapping[str, Any]] = None,
+    io_manager_key: Optional[str] = None,
+    save_notebook_on_failure: bool = False,
 ):
     """Wrap a Jupyter notebook in a op.
 
@@ -456,12 +353,20 @@ def define_dagstermill_op(
         tags (Optional[Dict[str, str]]): If set, additional tags used to annotate op.
             Dagster uses the tag keys `notebook_path` and `kind`, which cannot be
             overwritten by the user.
+        io_manager_key (Optional[str]): If using output_notebook_name, you can additionally provide
+            a string key for the IO manager used to store the output notebook.
+            If not provided, the default key output_notebook_io_manager will be used.
+        save_notebook_on_failure (bool): If True and the notebook fails during execution, the failed notebook will be
+            written to the Dagster storage directory. The location of the file will be printed in the Dagster logs.
+            Defaults to False.
 
     Returns:
         :py:class:`~dagster.OpDefinition`
     """
     check.str_param(name, "name")
     check.str_param(notebook_path, "notebook_path")
+    check.bool_param(save_notebook_on_failure, "save_notebook_on_failure")
+
     required_resource_keys = set(
         check.opt_set_param(required_resource_keys, "required_resource_keys", of_type=str)
     )
@@ -469,10 +374,13 @@ def define_dagstermill_op(
     ins = check.opt_mapping_param(ins, "ins", key_type=str, value_type=In)
 
     if output_notebook_name is not None:
-        required_resource_keys.add("output_notebook_io_manager")
+        io_mgr_key = check.opt_str_param(
+            io_manager_key, "io_manager_key", default="output_notebook_io_manager"
+        )
+        required_resource_keys.add(io_mgr_key)
         outs = {
             **outs,
-            cast(str, output_notebook_name): Out(io_manager_key="output_notebook_io_manager"),
+            cast(str, output_notebook_name): Out(io_manager_key=io_mgr_key),
         }
 
     if isinstance(asset_key_prefix, str):
@@ -487,11 +395,11 @@ def define_dagstermill_op(
     if tags is not None:
         check.invariant(
             "notebook_path" not in tags,
-            "user-defined solid tags contains the `notebook_path` key, but the `notebook_path` key is reserved for use by Dagster",
+            "user-defined op tags contains the `notebook_path` key, but the `notebook_path` key is reserved for use by Dagster",
         )
         check.invariant(
             "kind" not in tags,
-            "user-defined solid tags contains the `kind` key, but the `kind` key is reserved for use by Dagster",
+            "user-defined op tags contains the `kind` key, but the `kind` key is reserved for use by Dagster",
         )
     default_tags = {"notebook_path": notebook_path, "kind": "ipynb"}
 
@@ -503,6 +411,7 @@ def define_dagstermill_op(
             notebook_path,
             output_notebook_name,
             asset_key_prefix=asset_key_prefix,
+            save_notebook_on_failure=save_notebook_on_failure,
         ),
         ins=ins,
         outs=outs,
