@@ -1,11 +1,11 @@
 import os
 from pathlib import Path
-from typing import Any, List, Optional
+from typing import Any, Optional, Sequence
 
 import dagster._check as check
+from dagster import AssetKey, AssetMaterialization
 from dagster._config import Field
-from dagster._core.definitions.events import AssetKey
-from dagster._core.definitions.metadata import MetadataEntry, MetadataValue
+from dagster._core.definitions.metadata import MetadataValue
 from dagster._core.execution.context.input import InputContext
 from dagster._core.execution.context.output import OutputContext
 from dagster._core.storage.io_manager import IOManager, io_manager
@@ -13,14 +13,9 @@ from dagster._utils import mkdir_p
 
 
 class OutputNotebookIOManager(IOManager):
-    def __init__(self, asset_key_prefix: Optional[List[str]] = None):
+    def __init__(self, asset_key_prefix: Optional[Sequence[str]] = None):
         self.asset_key_prefix = asset_key_prefix if asset_key_prefix else []
 
-    def get_output_asset_key(self, context: OutputContext):
-        if context.has_asset_key:
-            return None
-        return AssetKey([*self.asset_key_prefix, f"{context.step_key}_output_notebook"])
-
     def handle_output(self, context: OutputContext, obj: bytes):
         raise NotImplementedError
 
@@ -31,7 +26,7 @@ class OutputNotebookIOManager(IOManager):
 class LocalOutputNotebookIOManager(OutputNotebookIOManager):
     """Built-in IO Manager for handling output notebook."""
 
-    def __init__(self, base_dir: str, asset_key_prefix: Optional[List[str]] = None):
+    def __init__(self, base_dir: str, asset_key_prefix: Optional[Sequence[str]] = None):
         super(LocalOutputNotebookIOManager, self).__init__(asset_key_prefix=asset_key_prefix)
         self.base_dir = base_dir
         self.write_mode = "wb"
@@ -54,11 +49,24 @@ class LocalOutputNotebookIOManager(OutputNotebookIOManager):
         mkdir_p(os.path.dirname(output_notebook_path))
         with open(output_notebook_path, self.write_mode) as dest_file_obj:
             dest_file_obj.write(obj)
-        yield MetadataEntry("notebook", value=MetadataValue.notebook(output_notebook_path))
+
+        metadata = {"Executed notebook": MetadataValue.notebook(output_notebook_path)}
+
+        if context.has_asset_key:
+            context.add_output_metadata(metadata)
+        else:
+            context.log_event(
+                AssetMaterialization(
+                    asset_key=AssetKey(
+                        [*self.asset_key_prefix, f"{context.step_key}_output_notebook"]
+                    ),
+                    metadata=metadata,
+                )
+            )
 
     def load_input(self, context) -> bytes:
         check.inst_param(context, "context", InputContext)
-        # pass output notebook to downstream solids as File Object
+        # pass output notebook to downstream ops as File Object
         with open(self._get_path(context.upstream_output), self.read_mode) as file_obj:
             return file_obj.read()
 
